@@ -2,8 +2,8 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
- *  Copyright (C) 2010 ST-Ericsson AB.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2010  ST-Ericsson AB.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -47,6 +47,7 @@ static const char *creg_prefix[] = { "+CREG:", NULL };
 static const char *cops_prefix[] = { "+COPS:", NULL };
 static const char *csq_prefix[] = { "+CSQ:", NULL };
 static const char *cind_prefix[] = { "+CIND:", NULL };
+static const char *zpas_prefix[] = { "+ZPAS:", NULL };
 static const char *option_tech_prefix[] = { "_OCTI:", "_OUWCTI:", NULL };
 
 struct netreg_data {
@@ -81,6 +82,40 @@ static void extract_mcc_mnc(const char *str, char *mcc, char *mnc)
 	mnc[OFONO_MAX_MNC_LENGTH] = '\0';
 }
 
+static int zte_parse_tech(GAtResult *result)
+{
+	GAtResultIter iter;
+	const char *network, *domain;
+	int tech;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+ZPAS:"))
+		return -1;
+
+	if (!g_at_result_iter_next_string(&iter, &network))
+		return -1;
+
+	if (!g_at_result_iter_next_string(&iter, &domain))
+		return -1;
+
+	if (g_str_equal(network, "GSM") == TRUE ||
+			g_str_equal(network, "GPRS") == TRUE)
+		tech = ACCESS_TECHNOLOGY_GSM;
+	else if (g_str_equal(network, "EDGE") == TRUE)
+		tech = ACCESS_TECHNOLOGY_GSM_EGPRS;
+	else if (g_str_equal(network, "UMTS") == TRUE)
+		tech = ACCESS_TECHNOLOGY_UTRAN;
+	else if (g_str_equal(network, "HSDPA") == TRUE)
+		tech = ACCESS_TECHNOLOGY_UTRAN_HSDPA;
+	else
+		tech = -1;
+
+	DBG("network %s domain %s tech %d", network, domain, tech);
+
+	return tech;
+}
+
 static int option_parse_tech(GAtResult *result)
 {
 	GAtResultIter iter;
@@ -169,6 +204,18 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, status, lac, ci, tech, cbd->data);
 }
 
+static void zte_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct ofono_netreg *netreg = cbd->data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+
+	if (ok)
+		nd->tech = zte_parse_tech(result);
+	else
+		nd->tech = -1;
+}
+
 static void option_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -215,10 +262,19 @@ static void at_registration_status(struct ofono_netreg *netreg,
 		g_at_chat_send(nd->chat, "AT$CNTI=0", none_prefix,
 				NULL, NULL, NULL);
 		break;
+	case OFONO_VENDOR_ZTE:
+		/*
+		 * Send +ZPAS? to find out the current tech, zte_tech_cb
+		 * will call, fire CREG? to do the rest.
+		 */
+		if (g_at_chat_send(nd->chat, "AT+ZPAS?", zpas_prefix,
+					zte_tech_cb, cbd, NULL) == 0)
+			nd->tech = -1;
+		break;
 	case OFONO_VENDOR_OPTION_HSO:
 		/*
 		 * Send AT_OCTI?;_OUWCTI? to find out the current tech,
-		 * option_tech_cb will call fire CREG? to do the rest.
+		 * option_tech_cb will call, fire CREG? to do the rest.
 		 */
 		if (g_at_chat_send(nd->chat, "AT_OCTI?;_OUWCTI?",
 					option_tech_prefix,
@@ -829,6 +885,34 @@ static void huawei_rssi_notify(GAtResult *result, gpointer user_data)
 				at_util_convert_signal_strength(strength));
 }
 
+static void huawei_mode_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	int mode, submode;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "^MODE:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &mode))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &submode))
+		return;
+
+	switch (mode) {
+	case 3:
+		nd->tech = ACCESS_TECHNOLOGY_GSM;
+		break;
+	case 5:
+		nd->tech = ACCESS_TECHNOLOGY_UTRAN;
+		break;
+	}
+}
+
 static void huawei_nwtime_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -1112,6 +1196,21 @@ static void cnti_query_tech_cb(gboolean ok, GAtResult *result,
 			tq->status, tq->lac, tq->ci, nd->tech);
 }
 
+static void zte_query_tech_cb(gboolean ok, GAtResult *result,
+						gpointer user_data)
+{
+	struct tech_query *tq = user_data;
+	int tech;
+
+	if (ok)
+		tech = zte_parse_tech(result);
+	else
+		tech = -1;
+
+	ofono_netreg_status_notify(tq->netreg,
+			tq->status, tq->lac, tq->ci, tech);
+}
+
 static void option_query_tech_cb(gboolean ok, GAtResult *result,
 						gpointer user_data)
 {
@@ -1161,6 +1260,11 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 					cnti_query_tech_cb, tq, g_free) > 0)
 			return;
 		break;
+	case OFONO_VENDOR_ZTE:
+		if (g_at_chat_send(nd->chat, "AT+ZPAS?", zpas_prefix,
+					zte_query_tech_cb, tq, g_free) > 0)
+			return;
+		break;
 	case OFONO_VENDOR_OPTION_HSO:
 		if (g_at_chat_send(nd->chat, "AT_OCTI?;_OUWCTI?",
 					option_tech_prefix,
@@ -1184,6 +1288,7 @@ static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *str;
+	char *signal_identifier = "signal";
 	int index;
 	int min = 0;
 	int max = 0;
@@ -1202,8 +1307,10 @@ static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	 * Telit encapsulates the CIND=? tokens with braces
 	 * so we need to skip them
 	 */
-	if (nd->vendor == OFONO_VENDOR_TELIT)
+	if (nd->vendor == OFONO_VENDOR_TELIT) {
 		g_at_result_iter_open_list(&iter);
+		signal_identifier = "rssi";
+	}
 
 	while (g_at_result_iter_open_list(&iter)) {
 		/* Reset invalid default value for every token */
@@ -1229,7 +1336,7 @@ static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		if (!g_at_result_iter_close_list(&iter))
 			goto error;
 
-		if (g_str_equal("signal", str) == TRUE) {
+		if (g_str_equal(signal_identifier, str) == TRUE) {
 			nd->signal_index = index;
 			nd->signal_min = min;
 			nd->signal_max = max;
@@ -1351,6 +1458,10 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_register(nd->chat, "^RSSI:", huawei_rssi_notify,
 						FALSE, netreg, NULL);
 
+		/* Register for system mode reports */
+		g_at_chat_register(nd->chat, "^MODE:", huawei_mode_notify,
+						FALSE, netreg, NULL);
+
 		/* Register for network time reports */
 		g_at_chat_register(nd->chat, "^NWTIME:", huawei_nwtime_notify,
 						FALSE, netreg, NULL);
@@ -1377,7 +1488,14 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 						NULL, NULL, NULL);
 		break;
 	case OFONO_VENDOR_ZTE:
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "+CTZV:", ctzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+CTZR=1", none_prefix,
+						NULL, NULL, NULL);
+		break;
 	case OFONO_VENDOR_NOKIA:
+	case OFONO_VENDOR_SAMSUNG:
 		/* Signal strength reporting via CIND is not supported */
 		break;
 	default:
